@@ -19,10 +19,8 @@
  */
 
 #include <config.h>
-
 #include <stdlib.h>
 #include <string.h>
-
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-utils.h>
@@ -32,8 +30,8 @@
 #include "gossip-jabber-utils.h"
 #include "gossip-jabber-private.h"
 
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_MSG(x)
+/*#define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 #define JOIN_TIMEOUT       20000
 
@@ -273,11 +271,13 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 {
 	GossipJID        *jid;
 	GossipMessage    *message;
+	GossipContact    *contact;
 	GossipChatroomId  id;
 	JabberChatroom   *room;
 	LmMessageNode    *node;
 	const gchar      *from;
 
+
 	if (lm_message_get_sub_type (m) != LM_MESSAGE_SUB_TYPE_GROUPCHAT) {
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
@@ -300,11 +300,8 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 					       "chatroom-new-room-event",
 					       id, node->value);
 		} else {
-			gossip_time_t  timestamp;
-			GossipContact *contact;
+			gossip_time_t timestamp;
 
-			contact = jabber_chatrooms_get_contact (room, jid, 
-								NULL);
 			timestamp = gossip_jabber_get_message_timestamp (m);
 
 			message = gossip_message_new (GOSSIP_MESSAGE_TYPE_CHAT_ROOM,
@@ -313,6 +310,8 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 			timestamp = gossip_jabber_get_message_timestamp (m);
 			gossip_message_set_timestamp (message, timestamp);
 			
+			contact = jabber_chatrooms_get_contact (room, jid, NULL);
+
 			gossip_message_set_sender (message, contact);
 			gossip_message_set_body (message, node->value);
 			
@@ -326,9 +325,11 @@ jabber_chatrooms_message_handler (LmMessageHandler      *handler,
 
 	node = lm_message_node_get_child (m->node, "subject");
 	if (node) {
+		contact = jabber_chatrooms_get_contact (room, jid, NULL);
+		
 		g_signal_emit_by_name (chatrooms->jabber,
-				       "chatroom-title-changed", 
-				       id, node->value);
+				       "chatroom-topic-changed", 
+				       id, contact, node->value);
 	}
 	
 	gossip_jid_unref (jid);
@@ -571,7 +572,7 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 	if (existing_room) {
  		jabber_chatrooms_chatroom_unref (room); 
 
-		/* duplicate room already exists */
+		/* Duplicate room already exists. */
 		id = jabber_chatrooms_chatroom_get_id (existing_room);
 
 		DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s', room already exists.", 
@@ -586,7 +587,7 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 		return id;
 	}
 
-	/* get real chatroom */
+	/* Get real chatroom. */
 	id = jabber_chatrooms_chatroom_get_id (room);
 
         DEBUG_MSG (("ProtocolChatrooms: ID[%d] Join chatroom:'%s' on server:'%s'", 
@@ -595,21 +596,21 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 		   gossip_chatroom_get_server (chatroom)));
 
 
-	/* add timeout for server response */
+	/* Add timeout for server response. */
 	room->timeout_id = g_timeout_add (JOIN_TIMEOUT, 
 					  (GSourceFunc) jabber_chatrooms_join_timeout_cb, 
 					  room);
 
 	room->chatrooms = chatrooms;
 
-	/* set callback data */
+	/* Set callback data. */
         room->callback = callback;
         room->user_data = user_data;
 	
 	gossip_chatroom_set_status (chatroom, GOSSIP_CHATROOM_STATUS_JOINING);
 	gossip_chatroom_set_last_error (room->chatroom, NULL);
 
-	/* compose message */
+	/* Compose message. */
 	m = lm_message_new_with_sub_type (gossip_jid_get_full (room->jid),
 					  LM_MESSAGE_TYPE_PRESENCE,
 					  LM_MESSAGE_SUB_TYPE_AVAILABLE);
@@ -621,7 +622,6 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 			     room->jid, 
 			     jabber_chatrooms_chatroom_ref (room));
 
-	/* release the local ref */
 	jabber_chatrooms_chatroom_unref (room); 
 
         show = gossip_jabber_presence_state_to_str (chatrooms->presence);
@@ -634,7 +634,7 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
         lm_message_node_set_attribute (m->node, "id", id_str);
         g_free (id_str);
 
-	/* send message */
+	/* Send message. */
 	room->connection = lm_connection_ref (chatrooms->connection);
         room->join_handler = lm_message_handler_new ((LmHandleMessageFunction) 
 						     jabber_chatrooms_join_cb,
@@ -645,8 +645,8 @@ gossip_jabber_chatrooms_join (GossipJabberChatrooms *chatrooms,
 						LM_MESSAGE_TYPE_PRESENCE,
 						LM_HANDLER_PRIORITY_FIRST);
         
-	/* some servers don't honor the id so we don't get a reply and
-	   are waiting forever */
+	/* Some servers don't honor the id so we don't get a reply and
+	   are waiting forever. */
  	lm_connection_send (chatrooms->connection, m,  NULL);
 
 	lm_message_unref (m);
@@ -664,8 +664,13 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 	GossipChatroomJoinResult  result;
 	GossipChatroomStatus      status;
 	GossipChatroomId          id;
+	GossipChatroomId          id_found;
 	LmMessageSubType          type;
 	LmMessageNode            *node = NULL;
+	const gchar              *from;
+	GossipJID                *jid;
+	JabberChatroom           *room_found;
+	gboolean                  room_match = FALSE;
 
 	if (!room || !room->join_handler) {
  		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS; 
@@ -673,6 +678,26 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 
 	chatrooms = room->chatrooms;
 
+	/* get room id */
+	id = jabber_chatrooms_chatroom_get_id (room);
+
+	from = lm_message_node_get_attribute (m->node, "from");
+	jid = gossip_jid_new (from);
+
+	room_found = g_hash_table_lookup (chatrooms->room_jid_hash, jid);
+	gossip_jid_unref (jid);
+
+	if (room_found) {
+		id_found = jabber_chatrooms_chatroom_get_id (room_found);
+		if (id == id_found) {
+			room_match = TRUE;
+		}
+	} 
+
+	if (!room_match) {
+ 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS; 
+	}
+
 	/* clean up the join timeout */
 	if (room->timeout_id) {
 		g_source_remove (room->timeout_id);
@@ -685,19 +710,15 @@ jabber_chatrooms_join_cb (LmMessageHandler *handler,
 		room->join_handler = NULL;
 	}
 
-	/* get room id */
-	id = jabber_chatrooms_chatroom_get_id (room);
-
         /* check for error */
-	type = lm_message_get_sub_type (m);
+ 	type = lm_message_get_sub_type (m);
 	if (type == LM_MESSAGE_SUB_TYPE_ERROR) {
 		node = lm_message_node_get_child (m->node, "error");
 	}
 	
 	if (node) {
 		const gchar *str;
-		gint         code;
-		
+		gint         code;		
 		str = lm_message_node_get_attribute (node, "code");
 		code = str ? atoi (str) : 0;
 		
@@ -781,7 +802,7 @@ gossip_jabber_chatrooms_cancel (GossipJabberChatrooms *chatrooms,
 	room = g_hash_table_lookup (chatrooms->room_id_hash, 
 				    GINT_TO_POINTER (id));
 	if (!room) {
-		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d\n", id);
+		g_warning ("ProtocolChatrooms: Unknown chatroom id: %d", id);
 		return;
 	}
 
