@@ -32,23 +32,28 @@
 #define GOSSIP_IS_CHAT_WINDOW_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_CHAT_WINDOW))
 #define GOSSIP_CHAT_WINDOW_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_CHAT_WINDOW, GossipChatWindowClass))
 
+
 typedef struct _GossipChatWindow GossipChatWindow;
 typedef struct _GossipChatWindowClass GossipChatWindowClass;
 typedef struct _GossipChatWindowPriv GossipChatWindowPriv;
 
+
 #include "gossip-chat.h"
 
+
 struct _GossipChatWindow {
         GObject parent;
         GossipChatWindowPriv *priv;
 };
 
+
 struct _GossipChatWindowClass {
         GObjectClass parent_class;
 };
 
+
 GType             gossip_chat_window_get_type        (void);
-GossipChatWindow *gossip_chat_window_get_default     (gboolean          for_group_chat);
+GossipChatWindow *gossip_chat_window_get_default     (void);
 
 GossipChatWindow *gossip_chat_window_new             (void);
 
@@ -61,7 +66,7 @@ void              gossip_chat_window_remove_chat     (GossipChatWindow *window,
 void              gossip_chat_window_switch_to_chat  (GossipChatWindow *window,
 						      GossipChat       *chat);
 
-gint              gossip_chat_window_get_group_chats (GossipChatWindow *window);
 gboolean          gossip_chat_window_has_focus       (GossipChatWindow *window);
 
+
 #endif /* __GOSSIP_CHAT_WINDOW_H__ */
